mod fm;

use std::fs;
use std::io;

use hound;

fn main()
{
    let wav_spec : hound::WavSpec = hound::WavSpec
    {
        channels        : 1,
        sample_rate     : fm::SAMPLE_RATE as u32,
        bits_per_sample : 32,
        sample_format   : hound::SampleFormat::Float
    };

    let mut wav_writer : hound::WavWriter<io::BufWriter<fs::File>> = hound::WavWriter::create("d:/fm.wav", wav_spec).unwrap();

    let mut wave_vector : Vec<f32> = vec![];

    let bpm         : usize  = 120;
    let note_length : isize = fm::calculate_note_length(bpm, 4);

    let parameter_array : [(f32, f32); 4] = [(4.5, 2.0), (7.0, 3.0), (3.0, 2.0), (11.0, 4.0)];

    for parameter in parameter_array
    {
        for scale in [60, 64, 67, 64, 60, 64, 67, 72]
        {
            fm::make_fm(&mut wave_vector, fm::Note { scale, length: note_length, gain: 0.5, params: parameter});
        }
    }

    for v in wave_vector.into_iter()
    {
        wav_writer.write_sample(v).unwrap();
    }
}